/*!
 * virtualized-list v2.0.0
 * MIT Licensed
 */
(function webpackUniversalModuleDefinition(root, factory) {
	if(typeof exports === 'object' && typeof module === 'object')
		module.exports = factory();
	else if(typeof define === 'function' && define.amd)
		define([], factory);
	else if(typeof exports === 'object')
		exports["VirtualizedList"] = factory();
	else
		root["VirtualizedList"] = factory();
})(this, function() {
return /******/ (function(modules) { // webpackBootstrap
/******/ 	// The module cache
/******/ 	var installedModules = {};

/******/ 	// The require function
/******/ 	function __webpack_require__(moduleId) {

/******/ 		// Check if module is in cache
/******/ 		if(installedModules[moduleId])
/******/ 			return installedModules[moduleId].exports;

/******/ 		// Create a new module (and put it into the cache)
/******/ 		var module = installedModules[moduleId] = {
/******/ 			exports: {},
/******/ 			id: moduleId,
/******/ 			loaded: false
/******/ 		};

/******/ 		// Execute the module function
/******/ 		modules[moduleId].call(module.exports, module, module.exports, __webpack_require__);

/******/ 		// Flag the module as loaded
/******/ 		module.loaded = true;

/******/ 		// Return the exports of the module
/******/ 		return module.exports;
/******/ 	}


/******/ 	// expose the modules object (__webpack_modules__)
/******/ 	__webpack_require__.m = modules;

/******/ 	// expose the module cache
/******/ 	__webpack_require__.c = installedModules;

/******/ 	// __webpack_public_path__
/******/ 	__webpack_require__.p = "";

/******/ 	// Load entry module and return exports
/******/ 	return __webpack_require__(0);
/******/ })
/************************************************************************/
/******/ ([
/* 0 */
/***/ function(module, exports, __webpack_require__) {

	module.exports = __webpack_require__(1);


/***/ },
/* 1 */
/***/ function(module, exports, __webpack_require__) {

	'use strict';

	exports.__esModule = true;
	exports.default = undefined;

	var _CellSizeAndPositionManager = __webpack_require__(2);

	var _CellSizeAndPositionManager2 = _interopRequireDefault(_CellSizeAndPositionManager);

	function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

	function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } } /** Virtualized list, only renders visible items.
	                                                                                                                                                           *  @param  {Array<*>} data           List of data items
	                                                                                                                                                           *  @param  {Function} renderRow      Renders a single row
	                                                                                                                                                           *  @param  {Number}   rowHeight      Static height of a row
	                                                                                                                                                           *  @param  {Number}   overscanCount  Amount of rows to render above and below visible area of the list
	                                                                                                                                                           *  @param  {Function} onMount        Callback that is invoked after the list has mounted
	                                                                                                                                                           *  @method {Function} scrollToIndex  Method used to scroll to any given index
	                                                                                                                                                           *
	                                                                                                                                                           *  @example
	                                                                                                                                                           *  new VirtualizedList(element, {
	                                                                                                                                                           *    data: ['a', 'b', 'c'],
	                                                                                                                                                           *    renderRow: (row, index) => { <div>{row}</div> },
	                                                                                                                                                           *    rowHeight: 150,
	                                                                                                                                                           *    overscanCount: 5
	                                                                                                                                                           *  )}
	                                                                                                                                                           *
	                                                                                                                                                           */

	var STYLE_INNER = 'position:relative; overflow:hidden; width:100%; min-height:100%;';
	var STYLE_CONTENT = 'position:absolute; top:0; left:0; height:100%; width:100%; overflow:visible;';

	var VirtualizedList = function () {
	  function VirtualizedList(container, options) {
	    var _this = this;

	    _classCallCheck(this, VirtualizedList);

	    this.container = container;
	    this.options = options;

	    // Initialization
	    this.state = {};
	    this._rowSizeAndPositionManager = new _CellSizeAndPositionManager2.default({
	      cellCount: options.data.length,
	      cellSizeGetter: function cellSizeGetter(_ref) {
	        var index = _ref.index;
	        return _this.getRowHeight(index);
	      },
	      estimatedCellSize: options.estimatedRowHeight || 100
	    });

	    // Binding
	    this.render = this.render.bind(this);
	    this.handleScroll = this.handleScroll.bind(this);
	    this.resize = this.resize.bind(this);

	    // Lifecycle Methods
	    this.componentWillMount();
	    this.componentDidMount();
	  }

	  VirtualizedList.prototype.componentWillMount = function componentWillMount() {
	    var inner = this.inner = document.createElement('div');
	    var content = this.content = document.createElement('div');

	    inner.setAttribute('style', STYLE_INNER);
	    content.setAttribute('style', STYLE_CONTENT);
	    inner.appendChild(content);
	    this.container.appendChild(inner);
	  };

	  VirtualizedList.prototype.componentDidMount = function componentDidMount() {
	    var onMount = this.options.onMount;

	    // Resize

	    this.resize(onMount);

	    // Add event listeners
	    this.container.addEventListener('scroll', this.handleScroll);
	    window.addEventListener('resize', this.resize);
	  };

	  VirtualizedList.prototype.setState = function setState() {
	    var _this2 = this;

	    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
	    var callback = arguments[1];

	    this.state = Object.assign(this.state, state);

	    window.requestAnimationFrame(function () {
	      _this2.render();

	      if (typeof callback === 'function') {
	        callback();
	      }
	    });
	  };

	  VirtualizedList.prototype.resize = function resize(callback) {
	    if (this.state.height !== this.container.offsetHeight) {
	      this.setState({
	        height: this.container.offsetHeight
	      }, callback);
	    }
	  };

	  VirtualizedList.prototype.handleScroll = function handleScroll() {
	    this.setState({
	      offset: this.container.scrollTop
	    });
	  };

	  VirtualizedList.prototype.getRowHeight = function getRowHeight(index) {
	    var rowHeight = this.options.rowHeight;


	    return Array.isArray(rowHeight) ? rowHeight[index] : rowHeight;
	  };

	  VirtualizedList.prototype.getRowOffset = function getRowOffset(index) {
	    var _rowSizeAndPositionMa = this._rowSizeAndPositionManager.getSizeAndPositionOfCell(index),
	        offset = _rowSizeAndPositionMa.offset;

	    return offset;
	  };

	  VirtualizedList.prototype.scrollToIndex = function scrollToIndex(index) {
	    var offset = this.getRowOffset(index);

	    this.container.scrollTop = offset;
	  };

	  VirtualizedList.prototype.destroy = function destroy() {
	    window.removeEventListener('resize', this.resize);

	    // TODO: Destroy everything properly, silly
	  };

	  VirtualizedList.prototype.getRowsForOffset = function getRowsForOffset(offset) {
	    var _options$overscanCoun = this.options.overscanCount,
	        overscanCount = _options$overscanCoun === undefined ? 3 : _options$overscanCoun;
	    var height = this.state.height;

	    var _rowSizeAndPositionMa2 = this._rowSizeAndPositionManager.getVisibleCellRange({
	      containerSize: height,
	      offset: offset
	    }),
	        start = _rowSizeAndPositionMa2.start,
	        stop = _rowSizeAndPositionMa2.stop;

	    if (overscanCount) {
	      start = Math.max(0, start - start % overscanCount);
	      stop += overscanCount;
	    }

	    return { start: start, stop: stop };
	  };

	  VirtualizedList.prototype.getTotalHeight = function getTotalHeight() {
	    return this._rowSizeAndPositionManager.getTotalSize();
	  };

	  VirtualizedList.prototype.render = function render() {
	    var _options = this.options,
	        data = _options.data,
	        renderRow = _options.renderRow;
	    var _state$offset = this.state.offset,
	        offset = _state$offset === undefined ? 0 : _state$offset;

	    var _getRowsForOffset = this.getRowsForOffset(offset),
	        start = _getRowsForOffset.start,
	        stop = _getRowsForOffset.stop;

	    var rows = data.slice(start, stop);
	    var fragment = document.createDocumentFragment();

	    rows.forEach(function (row, index) {
	      return fragment.append(renderRow(row, start + index));
	    });

	    this.inner.style.height = this.getTotalHeight() + 'px';
	    this.content.style.top = this.getRowOffset(start) + 'px';

	    this.content.innerHTML = '';
	    this.content.appendChild(fragment);
	  };

	  return VirtualizedList;
	}();

	exports.default = VirtualizedList;

/***/ },
/* 2 */
/***/ function(module, exports) {

	'use strict';

	exports.__esModule = true;

	function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

	/* Forked from https://github.com/bvaughn/react-virtualized/ 💖 */

	/**
	 * Just-in-time calculates and caches size and position information for a collection of cells.
	 */
	var CellSizeAndPositionManager = function () {
	  function CellSizeAndPositionManager(_ref) {
	    var cellCount = _ref.cellCount,
	        cellSizeGetter = _ref.cellSizeGetter,
	        estimatedCellSize = _ref.estimatedCellSize;

	    _classCallCheck(this, CellSizeAndPositionManager);

	    this._cellSizeGetter = cellSizeGetter;
	    this._cellCount = cellCount;
	    this._estimatedCellSize = estimatedCellSize;

	    // Cache of size and position data for cells, mapped by cell index.
	    // Note that invalid values may exist in this map so only rely on cells up to this._lastMeasuredIndex
	    this._cellSizeAndPositionData = {};

	    // Measurements for cells up to this index can be trusted; cells afterward should be estimated.
	    this._lastMeasuredIndex = -1;
	  }

	  CellSizeAndPositionManager.prototype.configure = function configure(_ref2) {
	    var cellCount = _ref2.cellCount,
	        estimatedCellSize = _ref2.estimatedCellSize;

	    this._cellCount = cellCount;
	    this._estimatedCellSize = estimatedCellSize;
	  };

	  CellSizeAndPositionManager.prototype.getCellCount = function getCellCount() {
	    return this._cellCount;
	  };

	  CellSizeAndPositionManager.prototype.getEstimatedCellSize = function getEstimatedCellSize() {
	    return this._estimatedCellSize;
	  };

	  CellSizeAndPositionManager.prototype.getLastMeasuredIndex = function getLastMeasuredIndex() {
	    return this._lastMeasuredIndex;
	  };

	  /**
	   * This method returns the size and position for the cell at the specified index.
	   * It just-in-time calculates (or used cached values) for cells leading up to the index.
	   */


	  CellSizeAndPositionManager.prototype.getSizeAndPositionOfCell = function getSizeAndPositionOfCell(index) {
	    if (index < 0 || index >= this._cellCount) {
	      throw Error('Requested index ' + index + ' is outside of range 0..' + this._cellCount);
	    }

	    if (index > this._lastMeasuredIndex) {
	      var lastMeasuredCellSizeAndPosition = this.getSizeAndPositionOfLastMeasuredCell();
	      var offset = lastMeasuredCellSizeAndPosition.offset + lastMeasuredCellSizeAndPosition.size;

	      for (var i = this._lastMeasuredIndex + 1; i <= index; i++) {
	        var size = this._cellSizeGetter({ index: i });

	        if (size == null || isNaN(size)) {
	          throw Error('Invalid size returned for cell ' + i + ' of value ' + size);
	        }

	        this._cellSizeAndPositionData[i] = {
	          offset: offset,
	          size: size
	        };

	        offset += size;
	      }

	      this._lastMeasuredIndex = index;
	    }

	    return this._cellSizeAndPositionData[index];
	  };

	  CellSizeAndPositionManager.prototype.getSizeAndPositionOfLastMeasuredCell = function getSizeAndPositionOfLastMeasuredCell() {
	    return this._lastMeasuredIndex >= 0 ? this._cellSizeAndPositionData[this._lastMeasuredIndex] : {
	      offset: 0,
	      size: 0
	    };
	  };

	  /**
	   * Total size of all cells being measured.
	   * This value will be completedly estimated initially.
	   * As cells as measured the estimate will be updated.
	   */


	  CellSizeAndPositionManager.prototype.getTotalSize = function getTotalSize() {
	    var lastMeasuredCellSizeAndPosition = this.getSizeAndPositionOfLastMeasuredCell();

	    return lastMeasuredCellSizeAndPosition.offset + lastMeasuredCellSizeAndPosition.size + (this._cellCount - this._lastMeasuredIndex - 1) * this._estimatedCellSize;
	  };

	  /**
	   * Determines a new offset that ensures a certain cell is visible, given the current offset.
	   * If the cell is already visible then the current offset will be returned.
	   * If the current offset is too great or small, it will be adjusted just enough to ensure the specified index is visible.
	   *
	   * @param align Desired alignment within container; one of "auto" (default), "start", or "end"
	   * @param containerSize Size (width or height) of the container viewport
	   * @param currentOffset Container's current (x or y) offset
	   * @param totalSize Total size (width or height) of all cells
	   * @return Offset to use to ensure the specified cell is visible
	   */


	  CellSizeAndPositionManager.prototype.getUpdatedOffsetForIndex = function getUpdatedOffsetForIndex(_ref3) {
	    var _ref3$align = _ref3.align,
	        align = _ref3$align === undefined ? 'auto' : _ref3$align,
	        containerSize = _ref3.containerSize,
	        currentOffset = _ref3.currentOffset,
	        targetIndex = _ref3.targetIndex;

	    if (containerSize <= 0) {
	      return 0;
	    }

	    var datum = this.getSizeAndPositionOfCell(targetIndex);
	    var maxOffset = datum.offset;
	    var minOffset = maxOffset - containerSize + datum.size;

	    var idealOffset = void 0;

	    switch (align) {
	      case 'start':
	        idealOffset = maxOffset;
	        break;
	      case 'end':
	        idealOffset = minOffset;
	        break;
	      case 'center':
	        idealOffset = maxOffset - (containerSize - datum.size) / 2;
	        break;
	      default:
	        idealOffset = Math.max(minOffset, Math.min(maxOffset, currentOffset));
	        break;
	    }

	    var totalSize = this.getTotalSize();

	    return Math.max(0, Math.min(totalSize - containerSize, idealOffset));
	  };

	  CellSizeAndPositionManager.prototype.getVisibleCellRange = function getVisibleCellRange(_ref4) {
	    var containerSize = _ref4.containerSize,
	        offset = _ref4.offset;

	    var totalSize = this.getTotalSize();

	    if (totalSize === 0) {
	      return {};
	    }

	    var maxOffset = offset + containerSize;
	    var start = this._findNearestCell(offset);

	    var datum = this.getSizeAndPositionOfCell(start);
	    offset = datum.offset + datum.size;

	    var stop = start;

	    while (offset < maxOffset && stop < this._cellCount - 1) {
	      stop++;

	      offset += this.getSizeAndPositionOfCell(stop).size;
	    }

	    return {
	      start: start,
	      stop: stop
	    };
	  };

	  /**
	   * Clear all cached values for cells after the specified index.
	   * This method should be called for any cell that has changed its size.
	   * It will not immediately perform any calculations; they'll be performed the next time getSizeAndPositionOfCell() is called.
	   */


	  CellSizeAndPositionManager.prototype.resetCell = function resetCell(index) {
	    this._lastMeasuredIndex = Math.min(this._lastMeasuredIndex, index - 1);
	  };

	  CellSizeAndPositionManager.prototype._binarySearch = function _binarySearch(_ref5) {
	    var high = _ref5.high,
	        low = _ref5.low,
	        offset = _ref5.offset;

	    var middle = void 0;
	    var currentOffset = void 0;

	    while (low <= high) {
	      middle = low + Math.floor((high - low) / 2);
	      currentOffset = this.getSizeAndPositionOfCell(middle).offset;

	      if (currentOffset === offset) {
	        return middle;
	      } else if (currentOffset < offset) {
	        low = middle + 1;
	      } else if (currentOffset > offset) {
	        high = middle - 1;
	      }
	    }

	    if (low > 0) {
	      return low - 1;
	    }
	  };

	  CellSizeAndPositionManager.prototype._exponentialSearch = function _exponentialSearch(_ref6) {
	    var index = _ref6.index,
	        offset = _ref6.offset;

	    var interval = 1;

	    while (index < this._cellCount && this.getSizeAndPositionOfCell(index).offset < offset) {
	      index += interval;
	      interval *= 2;
	    }

	    return this._binarySearch({
	      high: Math.min(index, this._cellCount - 1),
	      low: Math.floor(index / 2),
	      offset: offset
	    });
	  };

	  /**
	   * Searches for the cell (index) nearest the specified offset.
	   *
	   * If no exact match is found the next lowest cell index will be returned.
	   * This allows partially visible cells (with offsets just before/above the fold) to be visible.
	   */


	  CellSizeAndPositionManager.prototype._findNearestCell = function _findNearestCell(offset) {
	    if (isNaN(offset)) {
	      throw Error('Invalid offset ' + offset + ' specified');
	    }

	    // Our search algorithms find the nearest match at or below the specified offset.
	    // So make sure the offset is at least 0 or no match will be found.
	    offset = Math.max(0, offset);

	    var lastMeasuredCellSizeAndPosition = this.getSizeAndPositionOfLastMeasuredCell();
	    var lastMeasuredIndex = Math.max(0, this._lastMeasuredIndex);

	    if (lastMeasuredCellSizeAndPosition.offset >= offset) {
	      // If we've already measured cells within this range just use a binary search as it's faster.
	      return this._binarySearch({
	        high: lastMeasuredIndex,
	        low: 0,
	        offset: offset
	      });
	    } else {
	      // If we haven't yet measured this high, fallback to an exponential search with an inner binary search.
	      // The exponential search avoids pre-computing sizes for the full set of cells as a binary search would.
	      // The overall complexity for this approach is O(log n).
	      return this._exponentialSearch({
	        index: lastMeasuredIndex,
	        offset: offset
	      });
	    }
	  };

	  return CellSizeAndPositionManager;
	}();

	exports.default = CellSizeAndPositionManager;

/***/ }
/******/ ])
});
;